@@ -18,7 +18,7 @@ def authenticate():
             username = config.get('username')
             token = config.get('token')
             if username == '' or token == '':
-                print("You are not logged in yet. Use 'tport login'.")
+                print("You are not logged in yet. Use 'just login'.")
                 return
             return f(*args, **kwargs)
 
