@@ -1,25 +1,20 @@
 #!/usr/bin/env python3
 
-import six
-import json
 import os
 import sys
-import click
-import git
-import logging
-import click_log
-import functools
 import time
+import logging
+
+import click
+import click_log
 
 from clusterone import ClusteroneClient
 from clusterone import authenticate
+
 from clusterone.config import Config
 from clusterone.clusterone_client import MATRIX_URL
-
 from clusterone.version import VERSION
-from clusterone.utils import random_name_generator, normalize, normalize_string, select_valid_index, render_table, tokenize_repo, question, info, option, repo_name_validator, \
-    job_name_validator, select_repo, describe, select_job, \
-    select_valid_integer, get_current_version
+from clusterone.utils import select_valid_index, render_table, question, info, option, select_job
 from clusterone.tf_runner import run_tf
 
 from gettext import gettext as _
@@ -63,6 +58,7 @@ def cli(context):
     """
     Welcome to Clusterone Command Line Interface.
     """
+
     config = Config()
     config.load()
     client = ClusteroneClient(token=config.get('token'), username=config.get('username'))
@@ -75,10 +71,10 @@ def main():
     except ClusteroneException as exception:
         log_error(exception)
         sys.exit(exception.exit_code)
-        # except Exceptions as exception:
-        # # here Sentry logger is a good idea
-        #     pass
-
+   # TODO: THIS
+   # except Exceptions as exception:
+   # # here Sentry logger is a good idea
+   #     pass
 
 # ---------------------------------------
 
@@ -158,6 +154,8 @@ def start(config):
 from clusterone import commands
 
 get.add_command(commands.get.job.command, 'job')
+get.add_command(commands.get.events.command, 'events')
+
 create.add_command(commands.create.project.command, 'project')
 
 job.add_command(commands.create.job.single.command, 'single')
@@ -177,80 +175,15 @@ ln.add_command(commands.ln.dataset.command, 'dataset')
 start.add_command(commands.start.job.command, 'job')
 create.add_command(commands.create.dataset.command, 'dataset')
 
-# ---
+cli.add_command(commands.stop.job.command,'stop')
 
-cli.add_command(commands.stop.job.command, 'stop')
+cli.add_command(commands.login.command, 'login')
+cli.add_command(commands.logout.command, 'logout')
 
+cli.add_command(commands.matrix.command, 'matrix')
 
 # ------------------------
 
-@cli.command()
-@click.option('--username', '-u', prompt=True)
-@click.option('--password', '-p', prompt=True, hide_input=True)
-@click_log.simple_verbosity_option()
-@click_log.init(__name__)
-@pass_config
-def login(config, username, password):
-    """
-    Log into Clusterone
-    """
-
-    # Version Check
-    if not get_current_version():
-        click.echo(info("There is an updated version in PYPI - please run pip install clusterone --upgrade"))
-        return
-
-    client = ClusteroneClient()
-    user, message = client.api_login(username, password)
-    if user is not None:
-        # Save Token and Username in Config File
-        config['token'] = client.token
-        config['git_token'] = client.git_token
-        config['username'] = username
-        if user['first_name'] != '':
-            config['first_name'] = user['first_name']
-        else:
-            config['first_name'] = username
-
-        config.save()
-        click.echo(info("Welcome %s! \nYour last login was on %s" %
-                        (config['first_name'], user['last_login'])))
-
-        return client.token
-    else:
-        click.echo(info("Login Failed! %s" % message))
-        return
-
-
-@cli.command()
-@click.pass_obj
-@authenticate()
-def logout(context):
-    """
-    Log out from Clusterone
-    """
-
-    config = context.config
-
-    if click.confirm("%s, Are you sure you want to log out?" %
-                         (config.get('first_name'))):
-        config.delete()
-
-
-@cli.command()
-@click_log.simple_verbosity_option()
-@click_log.init(__name__)
-@pass_config
-def matrix(config):
-    """
-    Open Matrix in your browser
-    """
-    client = ClusteroneClient(
-        token=config.get('token'), username=config.get('username'))
-    click.echo(info("Entering Matrix: %s") % MATRIX_URL)
-    client.open_dashboard()
-
-
 @click.command()
 @click.option(
     '--owner', default=global_config.get('username'), help=owner_help_message)
@@ -301,92 +234,6 @@ def get_jobs(context, owner):
 
 get.add_command(get_jobs, 'jobs')
 
-
-@cli.command()
-@click_log.simple_verbosity_option()
-@click_log.init(__name__)
-@click.pass_obj
-@authenticate()
-def pulse(context):
-    """
-    Get status of jobs
-    """
-
-    config = context.config
-
-    # TODO: show resources for each job
-    client = ClusteroneClient(
-        token=config.get('token'), username=config.get('username'))
-    running_jobs = client.get_jobs()  # TODO Check running job parameter
-    if running_jobs:
-        click.echo(info('Jobs:'))
-        data = []
-        data.append(['#', 'Job', 'Project', 'Status', 'Launched at'])
-
-        i = 0
-        valid_jobs = []
-        for job in running_jobs:
-            try:
-                data.append([
-                    i,
-                    '%s/%s' % (job.get('owner'), job.get('display_name')),
-                    '%s/%s:%s' %
-                    (job.get('repository_owner'), job.get('repository_name'),
-                     job.get('git_commit_hash')[:8]),
-                    job.get('status'),
-                    '' if job.get('launched_at') is None else job.get(
-                        'launched_at')[:-5]
-                ])
-                i += 1
-                valid_jobs.append(job)
-            except:
-                pass
-        table = render_table(data, 36)
-        click.echo(table.table)
-        return running_jobs
-    else:
-        click.echo(info("You don't seem to have any jobs yet. Try 'tport create job' to make one."))
-        return
-
-
-@cli.command()
-@click_log.simple_verbosity_option()
-@click_log.init(__name__)
-@click.pass_obj
-@authenticate()
-def watch(context):
-    """
-    Shortcut for tport get events --watch
-    """
-
-    config = context.config
-
-    client = ClusteroneClient(
-        token=config.get('token'), username=config.get('username'))
-    while True:
-        events = client.get_events()
-        if events:
-            data = [['Time', 'Job', 'Status', 'Event']]
-
-            for e in events:
-                try:
-                    data.append([
-                        e.get('created_at')[:19],
-                        e.get('job_name'),
-                        e.get('event_level_display'),
-                        e.get('event_type_display')
-                    ])
-                except:
-                    pass
-            table = render_table(data, 54)
-            os.system("printf '\033c'")
-            click.echo(table.table)
-            time.sleep(1)
-        else:
-            click.echo("Failed to query recent events. Perhaps you have not started a job yet. Try 'tport run'.")
-            return None
-
-
 @click.command()
 @authenticate()
 @click.pass_obj
@@ -502,51 +349,8 @@ def get_datasets(context, owner=None):
 get.add_command(get_datasets, 'datasets')
 
 
-@click.command()
-@click.option('--watch', is_flag=True, default=False, help='Watch events every 1 second.')
-@click_log.simple_verbosity_option()
-@click_log.init(__name__)
-@click.pass_obj
-@authenticate()
-def get_events(context, watch):
-    """
-    See latest events
-    """
-    config = context.config
-
-    client = ClusteroneClient(
-        token=config.get('token'), username=config.get('username'))
-    flag = True
-    while flag:
-        flag = watch
-        events = client.get_events()
-        if events:
-            data = [['Time', 'Job', 'Status', 'Event']]
-
-            i = 0
-            for e in events:
-                try:
-                    data.append([
-                        e.get('created_at')[:19],
-                        e.get('job_name'),
-                        e.get('event_level_display'),
-                        e.get('event_type_display')
-                    ])
-                    i += 1
-                except:
-                    pass
-            table = render_table(data, 54)
-            if flag:
-                os.system("printf '\033c'")
-            click.echo(table.table)
-            if flag:
-                time.sleep(1)
-        else:
-            click.echo("Failed to query recent events. Perhaps you have not started a job yet. Try 'tport run'.")
-            return None
 
 
-get.add_command(get_events, 'events')
 
 
 # TODO: Manage this -> this code is from former run job command
@@ -597,9 +401,6 @@ def run_local_tf(package_path, module, training_mode, worker_replicas,
             text=question('Specify the python module to run'),
             default='main')
 
-    # Strip .py from module, many users leave it there!
-    module = module.strip('.py')
-
     if not current_env:
         if requirements is None:
             choice = select_valid_index(0, 2,
