@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
+
 import json
 import os
-import webbrowser
 import logging
 import click
 import coreapi
@@ -8,18 +9,39 @@ import py
 from coreapi import transports
 from coreapi.auth import TokenAuthentication
 
+# Python 2.7 compliance
+try:
+    from functools import lru_cache
+except ImportError as exception:
+    def identity(*args, **kwargs):
+        """
+        A dummy decorator that does nothing
+        """
+
+        def wrapper(function):
+            """
+            Like litteraly waste of space...
+            """
+            return function
+
+        return wrapper
+
+    lru_cache = identity
+
 from coreapi import Client
 from coreapi import codecs
 
 from raven import Client as RavenClient
 
 from clusterone import client_exceptions
+from clusterone.utilities import lazy_property, lazify
 
 raven_client = RavenClient(
     'https://4935170ea1284f9e8c3a0c7552cdb5bd:ac7d8158d3b24bc4aa4ea4d628c616e0@sentry.tensorport.com/5'
 )
 
 MATRIX_URL = 'https://clusterone.com/matrix'
+PYPI_VERSIONS_URL = 'https://pypi.python.org/pypi/clusterone/json'
 
 # API Endpoint
 API_URL = 'https://clusterone.com/api/'
@@ -28,21 +50,15 @@ API_URL = 'https://clusterone.com/api/'
 API_SCHEMA = 'https://clusterone.com/api/schema/'
 API_TOKEN = 'https://clusterone.com/api/token/'
 
-LOCAL_DEV = False
+#TODO: Get rid of LOCAL_DEV while refactoring this - use something more generic - ex. API target
+#Overwriting local dev value insted of variable, because it used across this module
+LOCAL_DEV = True if os.environ.get('JUST_LOCAL', 'False').upper() == "TRUE" else False
+
 if LOCAL_DEV:
-    # For Testing Purposes
     API_SCHEMA = 'http://127.0.0.1:8000/api/schema/'
     API_URL = 'http://127.0.0.1:8000/api/'
     MATRIX_URL = 'http://127.0.0.1:8000/'
 
-DEV = False
-if DEV:
-    # For Testing Purposes
-    API_SCHEMA = 'https://dev.clusterone.com/api/schema/'
-    API_URL = 'https://dev.clusterone.com/api/'
-    API_TOKEN = 'https://dev.clusterone.com/api/token/'
-    MATRIX_URL = 'https://dev.clusterone.com/matrix'
-
 
 def get_data_path(dataset_name, local_root, local_repo, path):
     """
@@ -61,13 +77,11 @@ def get_data_path(dataset_name, local_root, local_repo, path):
         if code runs on clusterone cloud this function returns:
             /data/dataset_name/path
     """
-    try:
-        #TODO: Check if this should be Tensorport_CLOUD ???
-        environment = os.environ['Clusterone_CLOUD']
-        if len(environment) > 0:
-            environment = 'clusterone-cloud'
-    except:
-        environment = 'local'
+    #TODO: DO THIS PROPPERLY!!!!!!
+    #environment = os.environ['CLUSTERONE_CLOUD']
+
+    environment = os.environ.get('CLUSTERONE_CLOUD') or os.environ.get('TENSORPORT_CLOUD')
+    environment = 'clusterone-cloud' if environment else "local"
 
     #TODO: Hotfixing TEN-819
     # Have no way of manually testing this
@@ -101,12 +115,11 @@ def get_logs_path(root):
     #TODO: See get_data_path() for more into, hotfixing TEN-819
     root = os.path.abspath(root)
 
-    try:
-        environment = os.environ['Clusterone_CLOUD']
-        if len(environment) > 0:
-            environment = 'clusterone-cloud'
-    except:
-        environment = 'local'
+    #TODO: DO THIS PROPPERLY!!!!!!
+    #environment = os.environ['CLUSTERONE_CLOUD']
+
+    environment = os.environ.get('CLUSTERONE_CLOUD') or os.environ.get('TENSORPORT_CLOUD')
+    environment = 'clusterone-cloud' if environment else "local"
 
     if environment == 'local':
         return os.path.join(root)
@@ -116,7 +129,6 @@ def get_logs_path(root):
         print("Environment is not valid. Assuming local.")
         return os.path.join(root)
 
-
 class ClusteroneClient(object):
     """
     Clusterone API Client
@@ -131,6 +143,37 @@ class ClusteroneClient(object):
     api_schema = {}
     use_in_memory_schema = True
 
+
+    #TODO: Test this!
+    @property
+    @lazify
+    def framework_slugs(self):
+        #TODO: Move this to function signature after removing 2.7 compliance
+        # type: () -> [str]
+
+        return [framework['slug'] for framework in self.get_frameworks()]
+
+
+    #TODO: Test this!
+    @property
+    @lazify
+    def instance_types_slugs(self):
+        #TODO: Move this to function signature after removing 2.7 compliance
+        # type: () -> [str]
+
+        return [machine['slug'] for machine in self.get_instance_types() if machine['show_for_workers']]
+
+
+    #TODO: Test this!
+    @property
+    @lazify
+    def ps_type_slugs(self):
+        #TODO: Move this to function signature after removing 2.7 compliance
+        # type: () -> [str]
+
+        return [machine['slug'] for machine in self.get_instance_types() if machine['show_for_ps']]
+
+
     def __init__(self, token="", username="", environment='', use_in_memory_schema=True, schema_json_path=None,
                  api_schema_url=API_SCHEMA, api_url=API_URL):
 
@@ -142,6 +185,7 @@ class ClusteroneClient(object):
         self.token = token
         self.username = username
 
+        # Load Schema from file or given path
         if not use_in_memory_schema:
             if schema_json_path:
                 self.document_path = schema_json_path
@@ -222,6 +266,19 @@ class ClusteroneClient(object):
                 self.set_schema(api_schema)
 
             return api_schema
+
+        except coreapi.exceptions.ErrorMessage as e:
+            # Reset token and try again
+            self.reset_client()
+            api_schema = self.api_client.get(API_SCHEMA)
+
+            if self.use_in_memory_schema:
+                self.api_schema = api_schema
+            else:
+                self.set_schema(api_schema)
+
+            return api_schema
+
         except Exception as e:
             print('Cannot download API Schema: %s' % str(e))
             raise
@@ -245,9 +302,10 @@ class ClusteroneClient(object):
         except coreapi.exceptions.LinkLookupError as exc:
             raise client_exceptions.LoginNotSupplied()
         except coreapi.exceptions.ErrorMessage as exc:
+            #TODO: Throtling, server error, gateway timeout, maintanance mode
             if 'non_field_errors' in exc.error and exc.error['non_field_errors']:
                 if (exc.error.title == '400 Bad Request') and exc.error._data['non_field_errors']._data[0] == 'Unable to log in with provided credentials.':
-                    print("Try again.")
+                    raise client_exceptions.LoginFailed()
             else:
                 if "400" in exc.error._title:
                     if "Enter a valid \"slug\" consisting of letters, numbers, underscores or hyphens." in exc.error._data["display_name"][0]:
@@ -268,9 +326,25 @@ class ClusteroneClient(object):
                         raise exc
 
             #TODO: Is this propper place for reporoting exceptions
-            raven_client.captureException()
+            #raven_client.captureException()
             return
 
+    def reset_client(self):
+        #TODO: Move this to function signature after removing 2.7 compliance
+        # type: () -> None
+        """
+        Used to reset token, schema and set empty transrport method for client.
+        Fixing CLUS-82.
+        """
+
+        self.token = None
+        self.transport = self.get_transport()
+        self.api_client = Client(
+            transports=[self.transport], decoders=self.decoders)
+
+        #TODO: Change to aquire schema in the future - to take advantage of cached schema and not making request
+        self.api_schema = self.download_schema()
+
     # -----------------------------
     # Authentication
     def api_login(self, username, password):
@@ -279,6 +353,7 @@ class ClusteroneClient(object):
                 print('Loading Initial Clusterone API Schema: %s' %
                       self.api_schema_url)
 
+            # Login to get token
             response = self.client_action(actions_keys=['token', 'create'], params={
                 'username': username,
                 'password': password,
@@ -310,21 +385,9 @@ class ClusteroneClient(object):
         except Exception as e:
             message = "Authentication was successful, some other error occurred when downloading API Schema: %s" % str(
                 e)
-            raven_client.captureException()
+            #raven_client.captureException()
             return None, message
 
-    def open_dashboard(self):
-        """
-        Opens Matrix in a browser
-        :return:
-        """
-        try:
-            browser = webbrowser.get("chrome")
-            browser.open(MATRIX_URL)
-        except:
-            print("If you're not using Chrome, we recommend it for best performance.")
-            webbrowser.open_new(MATRIX_URL)
-
     def create_dataset(self, display_name, description='Clusterone Dataset'):
         """
         Creates a dataset thru API
@@ -370,7 +433,7 @@ class ClusteroneClient(object):
                     raise client_exceptions.DuplicateProjectName()
                 else:
                     print("Couldn't create a project thru API, error: %s" % str(exception))
-                    raven_client.captureException()
+                    #raven_client.captureException()
                     return False
 
     def get_project(self, name, username=None):
@@ -392,7 +455,7 @@ class ClusteroneClient(object):
                 raise client_exceptions.NonExistantProject()
             else:
                 print("Couldn't fetch a project, error: %s" % str(e))
-                raven_client.captureException()
+                #raven_client.captureException()
                 return False
 
     def get_projects(self, owner=None, writer=None, reader=None):
@@ -415,7 +478,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't fetch a project, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def delete_project(self, name, username):
@@ -431,7 +494,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't delete a project, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def delete_dataset(self, name, username):
@@ -448,7 +511,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't delete a dataset, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def get_dataset(self, name, username):
@@ -467,7 +530,7 @@ class ClusteroneClient(object):
                 raise client_exceptions.NonExistantDataset()
             else:
                 print("Couldn't fetch a dataset, error: %s" % str(e))
-                raven_client.captureException()
+                #raven_client.captureException()
                 return False
 
     def get_datasets(self, owner=None, reader=None, writer=None):
@@ -486,7 +549,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't fetch a datasets, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def get_events(self):
@@ -495,14 +558,11 @@ class ClusteroneClient(object):
         :return: List of Jobs JSONs
         """
         # TODO: add various ownership levels
-        try:
-            response = self.client_action(
-                ['events', 'list'], params={}, validate=False)
-            return response
-        except Exception as e:
-            print("Couldn't fetch a events, error: %s" % str(e))
-            raven_client.captureException()
-            return False
+        # TODO: what can this return? - discuss with Michał -> potencial edge cases
+        # Error message: "Failed to query recent events. Perhaps you have not started a job yet. Try 'tport run'."
+        response = self.client_action(
+            ['events', 'list'], params={}, validate=False)
+        return response
 
     def create_event(self, job_id, event_type, event_level=20, event_content=None):
         """
@@ -519,9 +579,10 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't create event, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
+    @lru_cache(maxsize=1)
     def get_instance_types(self, params=None):
         """
         Get List of Instance Types
@@ -532,7 +593,7 @@ class ClusteroneClient(object):
                 ['instance-types', 'list'], params=params, validate=False)
             return response.get('results')
         except Exception as e:
-            raven_client.captureException()
+            #raven_client.captureException()
             print("Couldn't fetch list of instances, error: %s" % str(e))
 
     def get_frameworks(self, params=None):
@@ -547,7 +608,7 @@ class ClusteroneClient(object):
             return response.get('results')
         #TODO: WHY LEO, WHY?!!!!!!!
         except Exception as e:
-            raven_client.captureException()
+            #raven_client.captureException()
             print("Couldn't fetch list of frameworks, error: %s" % str(e))
 
     def get_tf_versions(self, params=None):
@@ -572,7 +633,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't fetch a project, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def get_job(self, params=None):
@@ -586,7 +647,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't fetch a job, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def create_job(self, name, description=None, parameters=None):
@@ -611,7 +672,7 @@ class ClusteroneClient(object):
             return response
         except Exception as error:
             print("Couldn't create a job, error: %s" % str(error))
-            raven_client.captureException()
+            #raven_client.captureException()
             raise
 
     def delete_job(self, job_id):
@@ -642,7 +703,7 @@ class ClusteroneClient(object):
                 raise client_exceptions._NonExistantJob()
 
             print("Couldn't start a job, error: %s" % str(exception))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def stop_job(self, job_id):
@@ -658,7 +719,7 @@ class ClusteroneClient(object):
             return response
         except Exception as e:
             print("Couldn't stop a job, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def get_file_list(self, job_id):
@@ -677,7 +738,7 @@ class ClusteroneClient(object):
                 return []
         except Exception as e:
             print("Couldn't find a job or files, error: %s" % str(e))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
 
     def download_file(self, job_id, filename):
@@ -694,5 +755,5 @@ class ClusteroneClient(object):
             return download
         except Exception as e:
             print("Download file %s error: %s" % (filename, str(e)))
-            raven_client.captureException()
+            #raven_client.captureException()
             return False
