@@ -5,4 +5,7 @@ from . import start
 from . import stop
 from . import get
 from . import rm
+from . import login
+from . import logout
+from . import matrix
 
