@@ -9,11 +9,9 @@ from clusterone.client_exceptions import RemoteAquisitionFailure
 
 def aquire_remote(name, config, client):
 
-    username = config['username']
+    username = config.get('username')
     retry_count, retry_interval = config.retry_count, config.retry_interval
 
-    #pytest.set_trace()
-
     for _ in range(retry_count):
         time.sleep(retry_interval)
 
