@@ -2,19 +2,15 @@ import click
 from click import Choice
 from click.exceptions import BadParameter
 
-from clusterone import authenticate
-
-from .base_cmd import base_options, base, WORKER_INSTANCE_TYPE
+from clusterone import client
+from clusterone.utilities import LazyChoice
+from .base_cmd import job_base_options, base
 
 
-@click.command()
-@click.pass_obj
-@authenticate()
-@base_options
-#TODO: Unifiy this with distributed at code level
+@job_base_options
 @click.option(
     '--instance-type',
-    type=WORKER_INSTANCE_TYPE,
+    type=LazyChoice(client.instance_types_slugs),
     default="t2.small",
     help="Type of single instance to run.")
 def command(context, **kwargs):
@@ -27,12 +23,11 @@ def command(context, **kwargs):
     job_configuration = base(context, kwargs)
 
     job_configuration['parameters']['mode'] = "single"
-
-    #TODO: This is crap, move this to base
-    job_configuration['parameters']['workers'] = {'slug': kwargs['instance_type'], 'replicas': 1}
-
-    if job_configuration['parameters']['framework'] == "pytorch"  and not job_configuration['parameters']['tf_version'] == "":
-        raise BadParameter("This pytorch version is not supported, try \"latest\" instead.", param_hint="--framework-version")
+    job_configuration['parameters']['workers'] = \
+    {
+        'slug': kwargs['instance_type'],
+        'replicas': 1
+    }
 
     client.create_job(
         job_configuration['meta']['name'],
