@@ -2,7 +2,7 @@ import click
 from click import confirm
 
 from clusterone import authenticate
-from clusterone.client_exceptions import BussyDatasetRemoveAttempt
+from clusterone.client_exceptions import BusyDatasetRemoveAttempt
 from clusterone.utilities import path_to_dataset
 
 
@@ -30,7 +30,7 @@ def command(context, dataset_path):
             jobs_using_this_dataset)
     )
     if dataset_used_by_running_job:
-        raise BussyDatasetRemoveAttempt()
+        raise BusyDatasetRemoveAttempt()
 
     confirm("Dummy?", abort=True)
 
