@@ -10,4 +10,6 @@ from .time_limit_to_minutes import main as time_limit_to_minutes
 from .make_table import main as make_table
 from .job_serializer import main as serialize_job
 
-from .lazy_choice import LazyChoice
+from .lazy_choice import LazyChoice, lazy_property, lazify
+from .version import is_latest_version
+
